from uuid import UUID

from pydantic import NonNegativeInt

from .base import BaseEngine


class DefaultEngine(BaseEngine):
    db = {}

    def __init__(self, _: str, table: str) -> None:
        self.table = table

    def _get(self, resource_id: UUID) -> dict:
        try:
            return self.db[resource_id]
        except KeyError as exception:
            raise self.exception(
                self.status.HTTP_404_NOT_FOUND, "not found") from exception

    async def db_insert_one(self, resource_id: UUID, resource_obj: dict) -> dict:
        self.db[resource_id] = resource_obj
        return {"id": resource_id, **resource_obj}

    async def db_select_one(self, resource_id: UUID) -> dict:
        return {"id": resource_id, **self._get(resource_id)}

    async def db_update_one(self, resource_id: UUID, resource_obj: dict) -> dict:
        self._get(resource_id)
        self.db[resource_id] = resource_obj
        return {"id": resource_id, **resource_obj}

    async def db_delete_one(self, resource_id: UUID) -> None:
        self._get(resource_id)
        self.db.pop(resource_id, None)
        return None

    async def db_select_all(self, skip: NonNegativeInt = 0, limit: NonNegativeInt = 100) -> list[dict]:
        return [{"id": k, **v} for k, v in self.db.items()][skip:limit]



from uuid import UUID

from pydantic import NonNegativeInt
from tinydb import Query, TinyDB

from .base import BaseEngine


class TinyDBEngine(BaseEngine):
    db: TinyDB
    path: str
    table: str

    def __init__(self, path: str, table: str) -> None:
        self.db = TinyDB(path)
        self.table = table

    def _get(self, resource_id: UUID, raise_exception: bool = True) -> dict:
        try:
            result = self.db.table(self.table).search(
                Query().resource_id == str(resource_id))
            assert result or not raise_exception, "not found"
        except AssertionError as exception:
            raise self.exception(self.status.HTTP_404_NOT_FOUND,
                                 exception.args[0]) from exception
        else:
            return result[0] if len(result) else {}

    async def db_insert_one(self, resource_id: UUID, resource_obj: dict) -> dict:
        self.db.table(self.table).insert(
            {"resource_id": str(resource_id), "resource_obj": resource_obj})
        return {"id": resource_id, **resource_obj}

    async def db_select_one(self, resource_id: UUID) -> dict:
        result = self._get(resource_id)
        return {"id": resource_id, **result["resource_obj"]}

    async def db_update_one(self, resource_id: UUID, resource_obj: dict) -> dict:
        self._get(resource_id)
        self.db.table(self.table).update(
            {"resource_obj": resource_obj}, Query().resource_id == str(resource_id))
        return {"id": resource_id, **resource_obj}

    async def db_delete_one(self, resource_id: UUID) -> None:
        self._get(resource_id)
        self.db.table(self.table).remove(
            Query().resource_id == str(resource_id))
        return

    async def db_select_all(self, skip: NonNegativeInt = 0, limit: NonNegativeInt = 100) -> list[dict]:
        return [{"id": result["resource_id"], **result["resource_obj"]} for result in self.db.table(self.table).all()][
            skip:limit
        ]



from pydantic import NonNegativeInt

from .base import BaseEngine, UUID, NoReturn


class PostgresEngine(BaseEngine):
    def __init__(self, path: str, table: str) -> None:
        pass

    async def db_insert_one(self, resource_id: UUID, resource_obj: dict) -> NoReturn:
        raise NotImplementedError

    async def db_select_one(self, resource_id: UUID) -> NoReturn:
        raise NotImplementedError

    async def db_update_one(self, resource_id: UUID, resource_obj: dict) -> NoReturn:
        raise NotImplementedError

    async def db_delete_one(self, resource_id: UUID) -> NoReturn:
        raise NotImplementedError

    async def db_select_all(self, skip: NonNegativeInt = 0, limit: NonNegativeInt = 100) -> NoReturn:
        raise NotImplementedError



from typing import Literal

from pydantic import dataclasses


@dataclasses.dataclass
class DBSpecification:
    """DB specification"""

    path: str
    engine: Literal["Default", "Postgres", "TinyDB"] = "Default"



import importlib
from typing import Any

from ..types import DBSpecification
from .base import BaseEngine


class DBEngine(BaseEngine):
    engine: BaseEngine

    def __init__(self, db_spec: DBSpecification, table: str) -> None:
        engine_module = importlib.import_module(f".{db_spec.engine.lower()}", package=__package__)
        engine_class: type[BaseEngine] = getattr(engine_module, f"{db_spec.engine}Engine")
        self.engine = engine_class(db_spec.path, table)

    async def db_insert_one(self, *args, **kwargs) -> Any:
        return await self.engine.db_insert_one(*args, **kwargs)

    async def db_select_one(self, *args, **kwargs) -> Any:
        return await self.engine.db_select_one(*args, **kwargs)

    async def db_update_one(self, *args, **kwargs) -> Any:
        return await self.engine.db_update_one(*args, **kwargs)

    async def db_delete_one(self, *args, **kwargs) -> Any:
        return await self.engine.db_delete_one(*args, **kwargs)

    async def db_select_all(self, *args, **kwargs) -> Any:
        return await self.engine.db_select_all(*args, **kwargs)
